unit ConfigMgr;
interface
uses Windows, SysUtils, Classes, Forms, INIFiles;

type TDirectory=record
                  Path:shortstring;
                  Recursive:boolean;
                end;
type DateString=string[8];

const AR43=4.0/3.0;
      AR169=16.0/9.0;
      EPSILON=1e-2;

var Directories:array of TDirectory;  // Verzeichnisse mit Bildern
    AutoSize:boolean;                 // Automatische Gre?
    FixedX,FixedY:integer;            // Manuelle Gre
    AspectRatio:double;               // Seitenverhltnis
    AutoZoom:integer;                 // Auto-Zoom (0 bis 10)
    DestFile:string;                  // Zieldatei
    Everytime:boolean;                // Bei jedem Start wechseln?
    LastUpdate:DateString;            // Datum des letzten Updates

var WinDir:string;
    INIFileName:string;
    Today:DateString;

procedure LoadConfig;
procedure ConfigToForm;
procedure FormToConfig;
procedure SaveConfig;

implementation
uses Main;

procedure DirFromString(i:integer;const Spec:string);
begin
  Directories[i].Path:=copy(Spec,2,length(Spec)-1);
  Directories[i].Recursive:=(Spec[1]='+');
end;

function DirToString(i:integer):string;
begin
  if Directories[i].Recursive
    then Result:='+'+Directories[i].Path
    else Result:='-'+Directories[i].Path;
end;

procedure LoadConfig;
var INI:TINIFile;
    DirIDs:TStringList;
    i:integer;
begin
  INI:=TINIFile.Create(INIFileName);
  with INI do begin
    AutoSize:=ReadBool('General','AutoSize',true);
    FixedX:=ReadInteger('General','FixedX',Screen.Width);
    FixedY:=ReadInteger('General','FixedY',Screen.Height);
    AspectRatio:=ReadFloat('General','AspectRatio',AR43);
    AutoZoom:=ReadInteger('General','AutoZoom',2);
    DestFile:=ReadString('General','DestFile',WinDir+'auto.bmp');
    Everytime:=ReadBool('General','Everytime',false);
    LastUpdate:=ReadString('General','LastUpdate','00000000');
    DirIDs:=TStringList.Create;
    ReadSection('Directories',DirIDs);
    SetLength(Directories,DirIDs.Count);
    for i:=0 to High(Directories) do
      DirFromString(i,ReadString('Directories',DirIDs[i],'-C:\'));
    DirIDs.Free;
    Free;
  end;
end;

procedure ConfigToForm;
var i:integer;
begin
  with MainForm do begin
    if ConfigMgr.AutoSize
      then CBAutoSize.Checked:=true
      else CBFixedSize.Checked:=true;
    AutoSizeChange(nil);
    EFixedX.Text:=IntToStr(FixedX);
    EFixedY.Text:=IntToStr(FixedY);
    if abs(AspectRatio-AR43)<=EPSILON then begin
      ARIs43.Checked:=true;
      EAspectRatio.Text:=FloatToStr(AR43);
    end else if abs(AspectRatio-AR43)<=EPSILON then begin
      ARIs169.Checked:=true;
      EAspectRatio.Text:=FloatToStr(AR169);
    end else begin
      ARCustom.Checked:=true;
      EAspectRatio.Text:=FloatToStr(AspectRatio);
    end;
    AspectRatioChange(nil);
    CAutoZoom.Position:=AutoZoom;
    EDestFile.Text:=DestFile;
    CBEverytime.Checked:=Everytime;
    DirectoryBox.Items.Clear;
    for i:=0 to High(Directories) do
      DirectoryBox.Items.Add(DirToString(i))
  end;
end;

procedure FormToConfig;
var i:integer;
begin
  with MainForm do begin
    ConfigMgr.AutoSize:=CBAutoSize.Checked;
    try FixedX:=StrToInt(EFixedX.Text); except FixedX:=Screen.Width; end;
    try FixedY:=StrToInt(EFixedY.Text); except FixedY:=Screen.Height; end;
    try AspectRatio:=StrToFloat(EAspectRatio.Text); except AspectRatio:=AR43; end;
    AutoZoom:=CAutoZoom.Position;
    DestFile:=EDestFile.Text;
    Everytime:=CBEverytime.Checked;
    SetLength(Directories,DirectoryBox.Items.Count);
    for i:=0 to High(Directories) do
      DirFromString(i,DirectoryBox.Items[i]);
  end;
end;

procedure SaveConfig;
var INI:TINIFile;
    i:integer;
begin
  INI:=TINIFile.Create(INIFileName);
  with INI do begin
    WriteBool('General','AutoSize',AutoSize);
    WriteInteger('General','FixedX',FixedX);
    WriteInteger('General','FixedY',FixedY);
    WriteFloat('General','AspectRatio',AspectRatio);
    WriteInteger('General','AutoZoom',AutoZoom);
    WriteString('General','DestFile',DestFile);
    WriteBool('General','Everytime',Everytime);
    if Everytime
      then DeleteKey('General','LastUpdate')
      else WriteString('General','LastUpdate',LastUpdate);
    EraseSection('Directories');
    for i:=0 to High(Directories) do
      WriteString('Directories','Dir'+IntToStr(i+1),DirToString(i));
    Free;
  end;
end;

procedure Init;
var WinDirBuf:array[0..MAX_PATH-1]of char; PWinDir:PChar;
    ST:TSystemTime;
begin
  PWinDir:=@WinDirBuf[0];
  GetWindowsDirectory(PWinDir,MAX_PATH);
  WinDir:=IncludeTrailingPathDelimiter(PWinDir);
  INIFileName:=WinDir+'WPchange.ini';
  GetLocalTime(ST);
  Today:=char(48+ST.wYear DIV 1000)+char(48+(ST.wYear DIV 100) MOD 10)+
         char(48+(ST.wYear DIV 10) MOD 10)+char(48+ST.wYear MOD 10)+
         char(48+ST.wMonth DIV 10)+char(48+ST.wMonth MOD 10)+
         char(48+ST.wDay DIV 10)+char(48+ST.wDay MOD 10);
end;
begin
  Init;
end.
