unit Core;
interface
uses Windows, SysUtils, Classes, Graphics, Forms, JPEG, INIFiles,
     ScaleBMP, ConfigMgr;

procedure DoIt;
procedure DailyUpdate;

implementation

const IMAGEGRANULARITY=64;
var Images:array of shortstring;
    ImageCount:integer;

procedure AddImage(const FileName:string);
begin
  if ImageCount>High(Images) then
    SetLength(Images,ImageCount+IMAGEGRANULARITY);
  Images[ImageCount]:=FileName;
  inc(ImageCount);
end;

var Extension:string;
procedure Collect(const Path:string;Recursive:boolean);
var SR:TSearchRec; Cont:integer;
begin
  Cont:=FindFirst(Path+'*.*',faAnyFile,SR);
  while Cont=0 do begin
    if (SR.Attr AND faDirectory=0) then begin
      Extension:=UpperCase(ExtractFileExt(SR.Name));
      if (Extension='.JPG') OR (Extension='.JPEG') OR (Extension='.BMP') then
        AddImage(Path+SR.Name); 
    end else begin
      if Recursive AND (SR.Name<>'.') AND (SR.Name<>'..') then
        Collect(Path+SR.Name+'\',Recursive);
    end;
    Cont:=FindNext(SR);
  end;
  FindClose(SR);
end;

procedure DoIt;
var i:integer;
    ITL:string;
    BMP,Scaled:TBitmap;
    JPEG:TJPEGImage;
    SizeX,SizeY,MaxX,MaxY:integer;
    BaseX,BaseY,NewX,NewY:integer;
    SourceAspect,Deviation:double;
    ScaleX,ScaleY:double;
begin
  SetLength(Images,0); ImageCount:=0;
  for i:=0 to High(Directories) do
    with Directories[i] do
      if DirectoryExists(Path) then
        Collect(IncludeTrailingPathDelimiter(Path),Recursive);
  ITL:=Images[random(ImageCount)];
  BMP:=TBitmap.Create;
  if UpperCase(ExtractFileExt(ITL))='.BMP' then
    BMP.LoadFromFile(ITL)
  else begin
    JPEG:=TJPEGImage.Create;
    JPEG.LoadFromFile(ITL);
    BMP.Assign(JPEG);
    JPEG.Free;
  end;
  SizeX:=BMP.Width; SizeY:=BMP.Height;
  SourceAspect:=SizeX/SizeY;
  if AutoSize then begin
    MaxX:=Screen.Width; MaxY:=Screen.Height;
  end else begin
    MaxX:=FixedX; MaxY:=FixedY;
  end;
  if abs(SourceAspect-AspectRatio)<=EPSILON then begin
    // SourceAspect ~ AspectRatio ==> direkt einsetzen
    NewX:=MaxX; NewY:=MaxY;
  end else begin
    // allgemeiner Fall: Wir stellen uns den Bildschirm mit normierter Gre
    //                   vor: Y-Gre=1.0, X-Gre=AspectRatio. Jetzt wird das
    //                   Bild entsprechend eingepat.
    // Abweichung der Seitenverhltnisse berechnen
    if SourceAspect>AspectRatio
      then Deviation:=SourceAspect/AspectRatio-1.0
      else Deviation:=AspectRatio/SourceAspect-1.0;
    if Deviation>AutoZoom*0.1 then begin
      // Abweichung zu gro fr AutoZoom: passend letterboxen
      ScaleX:=SourceAspect; ScaleY:=1.0;
      if ScaleX>AspectRatio then begin
        ScaleX:=AspectRatio; ScaleY:=AspectRatio/SourceAspect;
      end;
    end else if SourceAspect>AspectRatio then begin
      // Auto-Zoom: links+rechts abschneiden
      ScaleX:=SourceAspect; ScaleY:=1.0;
    end else begin
      // Auto-Zoom: oben+unten abschneiden
      ScaleX:=AspectRatio; ScaleY:=AspectRatio/SourceAspect;
    end;
    // Integer-Werte fr Skalierung ausrechnen
    NewX:=round(ScaleX*MaxX/AspectRatio);
    NewY:=round(ScaleY*MaxY);
  end;
  BaseX:=(MaxX-NewX)DIV 2;
  BaseY:=(MaxY-NewY)DIV 2;
  Scaled:=TBitmap.Create;
  Scaled.Width:=NewX;
  Scaled.Height:=NewY;
  Scale(BMP,Scaled);
  with BMP,Canvas do begin
    Width:=MaxX;
    Height:=MaxY;
    Brush.Style:=bsSolid;
    Brush.Color:=clBackground;
    FillRect(Rect(0,0,MaxX,MaxY));
    Draw(BaseX,BaseY,Scaled);
  end;
  Scaled.Free;
  try
    BMP.SaveToFile(DestFile);
    SystemParametersInfo(SPI_SETDESKWALLPAPER,0,PChar(DestFile),SPIF_UPDATEINIFILE);
  finally
    BMP.Free;
  end;
  LastUpdate:=Today;
end;

procedure DailyUpdate;
var INI:TINIFile;
begin
  if Everytime then begin DoIt; exit; end;
  if Today<>LastUpdate then begin
    DoIt;
    INI:=TINIFile.Create(INIFileName);
    INI.WriteString('General','LastUpdate',LastUpdate);
    INI.Free;
  end else
    SystemParametersInfo(SPI_SETDESKWALLPAPER,0,PChar(DestFile),SPIF_UPDATEINIFILE);
end;

end.
